﻿/*
======================================
JavaScripts of the AdSensor widget
- Author: Mikko Vestola
- Website: 
http://www.mvnet.fi/index.php?osio=Tietokoneet&sivu=Widgetit_-_AdSensor
- Email: mvnet@mvnet.fi

   ---------------------------------------------------
   
    This file is part of AdSensor widget.

    AdSensor widget is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    AdSensor widget is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AdSensor widget.  If not, see <http://www.gnu.org/licenses/>.	

   ---------------------------------------------------


======================================
*/

/* Do we print the debug messages to the console 
(at AdSense.kon must also be set <setting name="debug" value="on"/> at line 5) */
var debug = false;

// Here is saved the settings of the current skin in XML format (contents are read from XML file)
var skinXML;

// These are the email address and password that are required to login to AdSense
var login = "";
var passwd = "";

// This tells that is the current user succesfully logged in (e.g. maybe login has failed or user changed his login info)
var loginNotCompleted = true;

// This stores the time of the last page read (is needed to know if login is expired)
var lastPageReadTime = 0;

// Time in milliseconds when session expires (it seems to be that Google has set this to 30 minutes if no pages are load)
var sessionExpires = 30*60*1000;

// If parsing clicks and earnings is not successfull --> true
var parseFailed = false;

//At these variables are stored the values that you have earned at that period.
var todayEarnings = 0;
var yesterdayEarnings = 0;
var last7daysEarnings = 0;
var thismonthEarnings = 0;
var lastmonthEarnings = 0;
var sincelastpaymentEarnings = 0;

//At these variables are stored the click values per period
var todayClickVal = 0;
var yesterdayClickVal = 0;
var last7daysClickVal = 0;
var thismonthClickVal = 0;
var lastmonthClickVal = 0;
var sincelastpaymentClickVal = 0;

/* This keeps track that how many of the above values are not updated (update process is asyncronous
and we want to show when all values are updated. */
var updateCount = 6;
var updateCountOrig = 6; // Same as before but this is not changed (need to have to restore the original value)

// These are the addresses where the above earnings and clicks can be read (after the login is complete):
var todayURL = "https://www.google.com/adsense/report/overview?timePeriod=today";
var yesterdayURL = "https://www.google.com/adsense/report/overview?timePeriod=yesterday";
var last7daysURL = "https://www.google.com/adsense/report/overview?timePeriod=last7days";
var thismonthURL = "https://www.google.com/adsense/report/overview?timePeriod=thismonth";
var lastmonthURL = "https://www.google.com/adsense/report/overview?timePeriod=lastmonth";
var sincelastpaymentURL = "https://www.google.com/adsense/report/overview?timePeriod=sincelastpayment";

// URL-objects of the above URL strings
var todayURLObj = new URL();
var yesterdayURLObj = new URL();
var last7daysURLObj = new URL();
var thismonthURLObj = new URL();
var lastmonthURLObj = new URL();
var sincelastpaymentURLObj = new URL();

// This is the URL object of AdSense login form URL
var formURLObj;

// This is the URL object of the AdSense redirection URL (part of the login process)
var redirURLObj;

// This is the XMLHttpRequest object used to post the AdSense login form
var xmlhttp;

// The URL where is the adsense login form (login process requires a GA3T value which is got from cookies when this page is read)
var formURL = "https://www.google.com/accounts/ServiceLoginBox?service=adsense&ifr=true&rm=hide&fpui=3&nui=15&alwf=true&ltmpl=login&passive=true&continue=https%3A%2F%2Fwww.google.com%2Fadsense%2Fgaiaauth&followup=https%3A%2F%2Fwww.google.com%2Fadsense%2Fgaiaauth&hl=en_US";

// This is the URL of the AdSense authentication page where the fields of the login form are posted
var loginURL = "https://www.google.com/accounts/ServiceLoginBoxAuth";

// The value of the cookies are stored here, initialized to null
var cookies = null;

/* Here is saved the parsed GA3T value which is required for successful login 
Seems to be some sort of auto generated value which is got in headers when read the login form. 
This value is posted along other login form fields. Maybe checks that user uses the form to login? */
var GA3Tparsed = null;

/* The regular expression of the total earnings of given time period. 
This expression is searched at the todayURL, yesterdayURL etc. and it it is 
found, we can read the value of total earnings at that time period.
Note: The dot sign (.) does not match every character. It matches every character except
newline. It is better to use [\s\S] for matching any character since there might be
newlines in the HTML code that is parsed with this regular expression.*/
var earningsRegExp = /AdSense for Content[\s\S]*?<td[\s\S]*?>([\s\S]*?)<\/td>[\s\S]*?<td[\s\S]*?>([\s\S]*?)<\/td>[\s\S]*?Total Earnings[\s\S]*?\$([\s\S]*?)<\/td>/; 

/* The regular expression of login activation URL that is searched when login was successful (clicking the link confirms the login process). */
var redirectUrl = /<a[\s\S]*?href="([\s\S]*?)"[\s\S]*?>[\s\S]*?click[\s\S]*?here[\s\S]*?to[\s\S]*?continue/; 

/* The regular expression of the second login activation URL that is searched when login was successful (clicking the link confirms the login process). */
var redirectUrl2 = /<meta[\s\S]*?content[\s\S]*?url=(?:'|(?:&#39;))([\s\S]*?)(?:'|(?:&#39;))/; 


/*
 * Translates widget's text fields to user's language. 
 * Translations to different languages are found at Resources folder.
*/
function translate() {

	// Localize preferences
	preferenceGroups.login_grp.title = widget.getLocalizedString("login_grp_title");
	preferenceGroups.data_grp.title = widget.getLocalizedString("data_grp_title");
	preferenceGroups.look_grp.title = widget.getLocalizedString("look_grp_title");

	preferences.adsenseEmail.title = widget.getLocalizedString("email_title");
	preferences.adsenseEmail.description = widget.getLocalizedString("email_description");

	preferences.adsensePasswd.title = widget.getLocalizedString("passwd_title");
	preferences.adsensePasswd.description = widget.getLocalizedString("passwd_description");

	preferences.saveAdsensePassword.title = widget.getLocalizedString("saveAdsensePassword_title");
	preferences.saveAdsensePassword.description = widget.getLocalizedString("saveAdsensePassword_description");
		
	preferences.updateInterval.title = widget.getLocalizedString("updateInterval_title");
	preferences.updateInterval.description = widget.getLocalizedString("updateInterval_description");
	
	preferences.headsUp.title = widget.getLocalizedString("headsUp_title");
	preferences.headsUp.description = widget.getLocalizedString("headsUp_description");
	
	preferences.onlyManually.title = widget.getLocalizedString("onlyManually_title");
	preferences.onlyManually.description = widget.getLocalizedString("onlyManually_description");
	
	preferences.earningsSize.title = widget.getLocalizedString("earningsSize_title");
	preferences.earningsSize.description = widget.getLocalizedString("earningsSize_description");

	preferences.skin.title = widget.getLocalizedString("skin_title");
	preferences.skin.description = widget.getLocalizedString("skin_description");

	preferences.showGooglelogo.title = widget.getLocalizedString("showGooglelogo_title");
	preferences.showGooglelogo.description = widget.getLocalizedString("showGooglelogo_description");
	
	// Widget fields
	periodTitle.data = widget.getLocalizedString("periodTitle");
	clicksTitle.data = widget.getLocalizedString("clicksTitle");

	todayTitle.data = widget.getLocalizedString("todayTitle");
	yesterdayTitle.data = widget.getLocalizedString("yesterdayTitle");
	sevendaysTitle.data = widget.getLocalizedString("sevendaysTitle");
	thisMonthTitle.data = widget.getLocalizedString("thisMonthTitle");
	lastMonthTitle.data = widget.getLocalizedString("lastMonthTitle");
	lastpayTitle.data = widget.getLocalizedString("lastpayTitle");
	
	lastUpdated.data = widget.getLocalizedString("updating");
	
}

/*
 * Updates preferences: modifies widget's look by reading the setting of the current skin, also handle other settings
 */
 
function updatePreferences()
{
	// Make Google logo visible or hidden depending on set preference
	if (preferences.showGooglelogo.value == 1)
		googleLogo.visible=true;
	else
		googleLogo.visible=false;

	// Rear the skin XML file that is set in preferences and load the skin
 	skinXML = XMLDOM.parse(filesystem.readFile(preferences.skin.value));
	loadSkin();
	
	// If chose was that widget is updated only manually or only when heads up display is activated, stop timer
	if (preferences.headsUp.value == 1 || preferences.onlyManually.value == 1)
		timer1.ticking = false;
	else
		timer1.ticking = true;
		
	// Set timer interval (interval is seconds and preference values are minutes)
	timer1.interval = preferences.updateInterval.value*60;
	
	// Font sizes
	todayMoney.size = preferences.earningsSize.value;
	yesterdayMoney.size = preferences.earningsSize.value;
	sevendaysMoney.size = preferences.earningsSize.value;
	thisMonthMoney.size = preferences.earningsSize.value;
	lastMonthMoney.size = preferences.earningsSize.value;
	lastpayMoney.size = preferences.earningsSize.value;
	
	// Check that is login info changed, if it is changed, login is not completed by current user
	if ((login!="" && login != preferences.adsenseEmail.value) || 
		(passwd!="" && passwd != preferences.adsensePasswd.value))
		loginNotCompleted = true;
		
	// Set new login and password
	login = preferences.adsenseEmail.value;
	passwd = preferences.adsensePasswd.value;
		
	// If want to save password
	if (preferences.saveAdsensePassword.value != 1) {
		preferences.adsensePasswd.value = "";
		savePreferences();
	}
	
	// Login prompt: Ask email and password if both not given
	loginPrompt();
	
	// If login is not completed or login info was changed, login again
	if (loginNotCompleted)
		adsenseLogin();
	
}


/* This function reads the currently selected skin's settings from the skin's XML file.
It reads all the necessary elements and their settings and modifies the element's attributes
in widget. For example the URL of the background image is stored in skin.xml. If user chages
the current skin, this function reads the URL of the background image from the  current skin's XML file
and modifies widget so that the new background is visible. The function reads the following settings from
the XML file:
- Normal text element settings (like size and position). These elements are: AdSensor text and updating text
- Settings of the text elements of the result table's title row (the row which has titles like: Period, Clicks and $)
- Settings of the text elements of the result table (shows the clicks and income per period)
- Background image URL
- Refresh icon URL and position
- Google logo URL and position
*/
function loadSkin() {
	try {

		var i;  // Array iterator

		// Read all text elements from skin's XML file using XPath 1.0 (returns an array of all elements starting "<text>")
		var textElements = skinXML.evaluate("adsensor-skin/text");

		// Loop the array through and every time call a function with the elements's name in parameter
		for (i=0; i<textElements.length; i++) {
			updateSkinTextField(textElements.item(i).getAttribute("name"));
		}

		// Read all elements of title row from skin's XML file
		var titleRowElements = skinXML.evaluate("adsensor-skin/title");

		for (i=0; i<titleRowElements.length; i++) {
			updateSkinTableTitleRow(titleRowElements.item(i).getAttribute("name"));
		}

		// Read all elements of the table that shows the clicks, incomes and row titles
		var tableCells = skinXML.evaluate("adsensor-skin/cell");

		for (i=0; i<tableCells.length; i++) {
			updateSkinTableFields(tableCells.item(i).getAttribute("name"));
		}

		// Read and set backround image URL
		widget.getElementById("background").src = skinXML.evaluate("string(adsensor-skin/image[@name='background']/src)");

		// Read and set  refresh icon image URL and position
		widget.getElementById("refresh").src = skinXML.evaluate("string(adsensor-skin/image[@name='refresh']/src)");
		widget.getElementById("refresh").hOffset = skinXML.evaluate("string(adsensor-skin/image[@name='refresh']/hOffset)");
		widget.getElementById("refresh").vOffset = skinXML.evaluate("string(adsensor-skin/image[@name='refresh']/vOffset)");

		// Read  and set Google logo image URL and position
		widget.getElementById("googleLogo").src = skinXML.evaluate("string(adsensor-skin/image[@name='googleLogo']/src)");
		widget.getElementById("googleLogo").hOffset = skinXML.evaluate("string(adsensor-skin/image[@name='googleLogo']/hOffset)");
		widget.getElementById("googleLogo").vOffset = skinXML.evaluate("string(adsensor-skin/image[@name='googleLogo']/vOffset)");

	} catch( e ) {
		// Exception handling
		if (debug)
			log( e );
	}
}


/* This function reads the attribute values of the given text field from the current skin's XML file and updates them to widget. 
Used in regular text fields like AdSensor text and the "updating" text at the bottom.*/
function updateSkinTextField(fieldName) {

	// Get the text element in widget so we can update its attributes (size, position etc.)
	var element = widget.getElementById(fieldName);
	
	// Read and set the attributes of the element. Attribute values are stored in skin's XML file which are parsed with XPath 1.0
	element.size = skinXML.evaluate("string(adsensor-skin/text[@name='"+fieldName+"']/size)");
	element.font = skinXML.evaluate("string(adsensor-skin/text[@name='"+fieldName+"']/font)");
	element.style = skinXML.evaluate("string(adsensor-skin/text[@name='"+fieldName+"']/style)");
	element.hOffset = skinXML.evaluate("string(adsensor-skin/text[@name='"+fieldName+"']/hOffset)");
	element.vOffset = skinXML.evaluate("string(adsensor-skin/text[@name='"+fieldName+"']/vOffset)");
	element.hAlign = skinXML.evaluate("string(adsensor-skin/text[@name='"+fieldName+"']/hAlign)");
	element.vAlign = skinXML.evaluate("string(adsensor-skin/text[@name='"+fieldName+"']/vAlign)");
	element.color = skinXML.evaluate("string(adsensor-skin/text[@name='"+fieldName+"']/color)");

}


/* This function reads the attribute values of the given text field from the current skin's XML file and updates them to widget. 
Used in the text fields in the results table that show the clicks and income per period.*/
function updateSkinTableFields(fieldName) {

	// Get the text element in widget so we can update its attributes (size, position etc.)
	var element = widget.getElementById(fieldName);

	/* Read and set the attributes of the element. Attribute values are stored in skin's XML file which is parser with XPath 1.0.
	The attribute values depend on that is the text element part of the 1st, 2nd or 3rd column. This information is stored in skin's XML file.
	We want that all the text elements that are in the same column get the same size, alignment etc (so they are positioned at the same place
	at the vertical axis but not at the horizontal axis, since we have multiple rows).*/
	if (skinXML.evaluate("string(adsensor-skin/cell[@name='"+fieldName+"']/attribute::column)")=="first-column") {
		element.size = skinXML.evaluate("string(adsensor-skin/column[@name='first-column']/size)");
		element.font = skinXML.evaluate("string(adsensor-skin/column[@name='first-column']/font)");
		element.style = skinXML.evaluate("string(adsensor-skin/column[@name='first-column']/style)");
		element.hAlign = skinXML.evaluate("string(adsensor-skin/column[@name='first-column']/hAlign)");
		element.vAlign = skinXML.evaluate("string(adsensor-skin/column[@name='first-column']/vAlign)");
		element.hOffset = skinXML.evaluate("string(adsensor-skin/column[@name='first-column']/hOffset)");
		element.color = skinXML.evaluate("string(adsensor-skin/column[@name='first-column']/color)");
	} else if (skinXML.evaluate("string(adsensor-skin/cell[@name='"+fieldName+"']/attribute::column)")=="second-column") {
		element.size = skinXML.evaluate("string(adsensor-skin/column[@name='second-column']/size)");
		element.font = skinXML.evaluate("string(adsensor-skin/column[@name='second-column']/font)");
		element.style = skinXML.evaluate("string(adsensor-skin/column[@name='second-column']/style)");
		element.hAlign = skinXML.evaluate("string(adsensor-skin/column[@name='second-column']/hAlign)");
		element.vAlign = skinXML.evaluate("string(adsensor-skin/column[@name='second-column']/vAlign)");
		element.hOffset = skinXML.evaluate("string(adsensor-skin/column[@name='second-column']/hOffset)");
		element.color = skinXML.evaluate("string(adsensor-skin/column[@name='second-column']/color)");
	}  else if (skinXML.evaluate("string(adsensor-skin/cell[@name='"+fieldName+"']/attribute::column)")=="third-column") {
		element.size = skinXML.evaluate("string(adsensor-skin/column[@name='third-column']/size)");
		element.font = skinXML.evaluate("string(adsensor-skin/column[@name='third-column']/font)");
		element.style = skinXML.evaluate("string(adsensor-skin/column[@name='third-column']/style)");
		element.hAlign = skinXML.evaluate("string(adsensor-skin/column[@name='third-column']/hAlign)");
		element.vAlign = skinXML.evaluate("string(adsensor-skin/column[@name='third-column']/vAlign)");
		element.hOffset = skinXML.evaluate("string(adsensor-skin/column[@name='third-column']/hOffset)");
		element.color = skinXML.evaluate("string(adsensor-skin/column[@name='third-column']/color)");
	}

	/* Check that in what row is the text element and set the vertical offset attribute to accordin value (every row has different value).
	We want that all the text elements that are set to the same row in skin's XML file get the same vOffset (so they are at the same horizontal line). */
	if (skinXML.evaluate("string(adsensor-skin/cell[@name='"+fieldName+"']/attribute::row)")=="first-row")
		element.vOffset = skinXML.evaluate("string(adsensor-skin/row[@name='first-row']/vOffset)");
	else if (skinXML.evaluate("string(adsensor-skin/cell[@name='"+fieldName+"']/attribute::row)")=="second-row")
		element.vOffset = skinXML.evaluate("string(adsensor-skin/row[@name='second-row']/vOffset)");
	else if (skinXML.evaluate("string(adsensor-skin/cell[@name='"+fieldName+"']/attribute::row)")=="third-row")
		element.vOffset = skinXML.evaluate("string(adsensor-skin/row[@name='third-row']/vOffset)");
	else if (skinXML.evaluate("string(adsensor-skin/cell[@name='"+fieldName+"']/attribute::row)")=="fourth-row")
		element.vOffset = skinXML.evaluate("string(adsensor-skin/row[@name='fourth-row']/vOffset)");
	else if (skinXML.evaluate("string(adsensor-skin/cell[@name='"+fieldName+"']/attribute::row)")=="fifth-row")
		element.vOffset = skinXML.evaluate("string(adsensor-skin/row[@name='fifth-row']/vOffset)");
	else if (skinXML.evaluate("string(adsensor-skin/cell[@name='"+fieldName+"']/attribute::row)")=="sixth-row")
		element.vOffset = skinXML.evaluate("string(adsensor-skin/row[@name='sixth-row']/vOffset)");

}


/* This function reads the attribute values of the given text field from the current skin's XML file and updates them to widget. 
Used in the text fields in the title row of the results table (column titles like, Period, Clicks and dollar sign).*/
function updateSkinTableTitleRow(fieldName) {

	// Get the text element in widget so we can update its attributes (size, position etc.)
	var element = widget.getElementById(fieldName);

	/* Read and set the attributes of the element. Attribute values are stored in skin's XML file which is parser with XPath 1.0.
	The attribute values depend on that is the text element part of the 1st, 2nd or 3rd column. This information is stored in skin's XML file.
	We want that all the text elements in the title row that are in the same column get the same horizontal alignment (so they are positioned at the same place
	at the horizontal axis but not at the vertical axis, since we have multiple colums).*/
	if (skinXML.evaluate("string(adsensor-skin/title[@name='"+fieldName+"']/attribute::column)")=="first-column") {
		element.hAlign = skinXML.evaluate("string(adsensor-skin/column[@name='first-column']/hAlign)");
		element.hOffset = skinXML.evaluate("string(adsensor-skin/column[@name='first-column']/hOffset)");
	} else if (skinXML.evaluate("string(adsensor-skin/title[@name='"+fieldName+"']/attribute::column)")=="second-column") {
		element.hAlign = skinXML.evaluate("string(adsensor-skin/column[@name='second-column']/hAlign)");
		element.hOffset = skinXML.evaluate("string(adsensor-skin/column[@name='second-column']/hOffset)");
	}  else if (skinXML.evaluate("string(adsensor-skin/title[@name='"+fieldName+"']/attribute::column)")=="third-column") {
		element.hAlign = skinXML.evaluate("string(adsensor-skin/column[@name='third-column']/hAlign)");
		element.hOffset = skinXML.evaluate("string(adsensor-skin/column[@name='third-column']/hOffset)");
	}

	/* The font size and everything else positioning and font style info is stored in skin's XML file for the title row. 
	Read them and modify element's attributes in widget.*/
	element.size = skinXML.evaluate("string(adsensor-skin/row[@name='title-row']/size)");
	element.font = skinXML.evaluate("string(adsensor-skin/row[@name='title-row']/font)");
	element.style = skinXML.evaluate("string(adsensor-skin/row[@name='title-row']/style)");
	element.vAlign = skinXML.evaluate("string(adsensor-skin/row[@name='title-row']/vAlign)");
	element.color = skinXML.evaluate("string(adsensor-skin/row[@name='title-row']/color)");
	element.vOffset = skinXML.evaluate("string(adsensor-skin/row[@name='title-row']/vOffset)");

}


/* Update when heads up display is activated (and manual update is not chosen) */
function konsposeActiv() {
	if (preferences.onlyManually.value!=1 && preferences.headsUp.value==1)
		updateAllEarnings();
}

/* Updates all values when timer's clock has been fired (if manual update is not chosen) */
function timerFired() {
	if (preferences.onlyManually.value!=1 && preferences.headsUp.value!=1)
		updateAllEarnings();
}

/* This function prompts user's email and password if no such variables hasn't been stored to preferences */
function loginPrompt() {

	if (login != "" && passwd != "") {
		// OK, there were saved login info at widget preferences
	} else {
		
		// Make the form
		var formfields = Array();
		
		formfields[0] =	new FormField();
		formfields[0].name = 'email';
		formfields[0].type = 'text';
		formfields[0].defaultValue = preferences.adsenseEmail.value;
		formfields[0].title = widget.getLocalizedString("email_title");
		formfields[0].description = widget.getLocalizedString("email_description");

		formfields[1] =	new FormField();
		formfields[1].name = 'passwd';
		formfields[1].type = 'text';
		formfields[1].secure = 'yes';	
		formfields[1].defaultValue = preferences.adsensePasswd.value;
		formfields[1].title = widget.getLocalizedString("passwd_title");
		formfields[1].description = widget.getLocalizedString("passwd_description");

		formfields[2] =	new FormField();
		formfields[2].name = 'savePwd';
		formfields[2].type = 'checkbox';
		formfields[2].defaultValue = preferences.saveAdsensePassword.value;
		formfields[2].title = widget.getLocalizedString("saveAdsensePassword_title");
		formfields[2].description = widget.getLocalizedString("saveAdsensePassword_description");
		
		// Show the form
		formResults = form(formfields, widget.getLocalizedString("loginPromptTitle"));
		
		// Check if cancel button was pressed (formResults == null)
		if (formResults != null) {
							
			login = formResults[0];
			passwd = formResults[1];
			
			// Check that values are correct and save them (password is not saved if user wanted that)
			if (login!="" && passwd!="") {
				preferences.adsenseEmail.value = login;
				preferences.saveAdsensePassword.value = formResults[2];
				if (formResults[2]==1)
					preferences.adsensePasswd.value = passwd;
				savePreferences();
			} else {
				// Alert user that values must be correct
				var answer = alert(widget.getLocalizedString("loginPromptFail"), 
				widget.getLocalizedString("loginPromptEnter"), 
				widget.getLocalizedString("loginPromptClose"));

				if (answer == 2)
					closeWidget();
				else
					loginPrompt();
			}
			
		} else {
			// Alert user that this widget needs correct email and password
			var answer = alert(widget.getLocalizedString("loginPromptFail"), 
			widget.getLocalizedString("loginPromptEnter"), 
			widget.getLocalizedString("loginPromptClose"));
			
			if (answer == 2)
				closeWidget();
			else
				loginPrompt();			
		}
			
	}


}

/* This function updates the "last updated" field (sets current date and time) */
function updateLastupdated() {

	// Calculate time now
	var currentTime = new Date();
	var month = currentTime.getMonth() + 1;
	var day = currentTime.getDate();
	var year = currentTime.getFullYear();
	var hour = currentTime.getHours();
	var min = currentTime.getMinutes();
	var sec = currentTime.getSeconds();
	
	var dayNumber = currentTime.getDay();
	var dayName = "";
	
	if (dayNumber==1)
		dayName = widget.getLocalizedString("day_Mon");
	else if (dayNumber==2)
		dayName = widget.getLocalizedString("day_Tue");
	else if (dayNumber==3)
		dayName = widget.getLocalizedString("day_Wed");
	else if (dayNumber==4)
		dayName = widget.getLocalizedString("day_Thu");
	else if (dayNumber==5)
		dayName = widget.getLocalizedString("day_Fri");
	else if (dayNumber==6)
		dayName = widget.getLocalizedString("day_Sat");
	else if (dayNumber==0)
		dayName = widget.getLocalizedString("day_Sun");

	// We want always to digits to hours, minutes and seconds
	if (hour < 10)
		hour = "0"+hour;
	if (min < 10)
		min = "0"+min;
	if (sec < 10)
		sec = "0"+sec;
	
	// Update field
	lastUpdated.data = widget.getLocalizedString("lastUpdated")+dayName+" "+
	day+"."+month+"."+year+" "+hour+":"+min+":"+sec;

	
}

/* This function clears all URL objects that are used to fetch clicks and earnings from Google.
All these URL objects must be cleared before reloading these URLs (updating with or without new login). 
Otherwise it seems to be that old pages are read and the login process fails. */
function initializeURLs() {

	// Clear all previous data
	todayURLObj.clear();
	yesterdayURLObj.clear();
	last7daysURLObj.clear();
	thismonthURLObj.clear();
	lastmonthURLObj.clear();
	sincelastpaymentURLObj.clear();

	// Specify correct addressess
	todayURLObj.location = todayURL;
	yesterdayURLObj.location = yesterdayURL;
	last7daysURLObj.location = last7daysURL;
	thismonthURLObj.location = thismonthURL;
	lastmonthURLObj.location = lastmonthURL;
	sincelastpaymentURLObj.location = sincelastpaymentURL;
	
	// Make all earnings and clicks a little trasparent when updating data
	fadeOut(todayMoney);
	fadeOut(yesterdayMoney);
	fadeOut(sevendaysMoney);
	fadeOut(thisMonthMoney);
	fadeOut(lastMonthMoney);
	fadeOut(lastpayMoney);

	fadeOut(todayClicks);
	fadeOut(yesterdayClicks);
	fadeOut(sevendaysClicks);
	fadeOut(thisMonthClicks);
	fadeOut(lastMonthClicks);
	fadeOut(lastpayClicks);

}

/* This makes a text object more transparent */
function fadeOut(textObject) {
	animator.start(new FadeAnimation(textObject, 100, 600, animator.kEaseOut));
}

/* This removes the transparency from the text object */
function fadeIn(textObject) {
	animator.start(new FadeAnimation(textObject, 255, 1000, animator.kEaseOut));
}

/* This sets the refresh icon onmouseover state */
function refreshIconOver() {
	// Update the refresh icon
	refresh.src = "Images/refresh_mouseover.png";
}

/* This sets the refresh icon to normal state */
function refreshIconExit() {
	// Update the refresh icon
	refresh.src = "Images/refresh.png";
}

/* This sets the refresh icon back to onmouseover state (or leaves it to normal state) when user has released the mouse button */
function refreshIconUp() {
	// Update the refresh icon
	if (refresh.src != "Images/refresh.png")
		refresh.src = "Images/refresh_mouseover.png";
}


/* This function updates all the click and earning values show at widget. 
This function is excecuted when user presses the refresh button at top right corner. */
function updateAllEarnings() {

	var d = new Date(); // Time now

	// Update the refresh icon (if clause is necessary because of automatic update)
	if (refresh.src != "Images/refresh.png")
		refresh.src = "Images/refresh_mouseclick.png";
	
	/* Do login, if current user has not been logged in (e.g. user has changed his login info) 
	or the session has expired. */
	if (loginNotCompleted || (d.getTime()-lastPageReadTime)>=sessionExpires) {
		// Show user that login is in process
		lastUpdated.data = widget.getLocalizedString("loggingIn");
		
		adsenseLogin();
		return;
	}
		
	// Show user that data is being updated
	lastUpdated.data = widget.getLocalizedString("updating");
	
	// Set updatecount to original updatecount so we know when all values have been updated
	updateCount = updateCountOrig;

	// Clear all previous fetches
	initializeURLs();

	// Fetch every page again
	readPage(todayURLObj);
	readPage(yesterdayURLObj);
	readPage(last7daysURLObj);
	readPage(thismonthURLObj);
	readPage(lastmonthURLObj);
	readPage(sincelastpaymentURLObj);
}


/* This reads all earnings (and clicks) after login is successfully completed. Used ONLY after successfull login. */
function readAllEarnings(urlObj) {

	/* If the login is not completed (so user has logged in with Google account and we have found
	a redirect url from the page), finalize the login process.*/
	if (urlObj.responseData.search(redirectUrl2)!=-1) {
				
		// Read activation URL and replace HTML ampersand marks
		var redirURL = RegExp.$1;
		redirURL = redirURL.replace(/&amp;/g,"&");
						
		if (debug) {
			log("Redir URL 2:\n"+redirURL+"\n\n\n");
		}					

		// Finalize the login and return (we don't want to execute the code below until login is completely finished
		finalizeLogin(redirURL);
		return;
	}

	// Login succesfully completed
	loginNotCompleted=false;

	// Store cookies after successful login
	storeCookies(urlObj.getResponseHeaders("Set-Cookie"));
			
	if (debug) {
		log("Login completed.\n\n\n");
		log("Cookies after login finish:\n"+cookies+"\n\n\n");
		log("Page after login finish:\n"+urlObj.responseData+"\n\n\n");
	}
	
	// Show user that data is being updated
	lastUpdated.data = widget.getLocalizedString("updating");
		
	// Set updatecount to original updatecount so we know when all values have been updated
	updateCount = updateCountOrig;

	// Clear all previous fetches
	initializeURLs();
	
	// Fetch every page again
	readPage(todayURLObj);
	readPage(yesterdayURLObj);
	readPage(last7daysURLObj);
	readPage(thismonthURLObj);
	readPage(lastmonthURLObj);
	readPage(sincelastpaymentURLObj);	
}


/* This function sets the clicks and earnigs for the given time period.
The time period is got from the urlObj location. */
function setClicksAndEarnings(urlObj, clicks, earnings) {

	// Trim all leading and trailing whitespaces and update data to right field in widget
	if (urlObj.location==todayURL) {
		todayClickVal = trim(clicks);
		todayEarnings = trim(earnings);
		todayMoney.data = todayEarnings;
		todayClicks.data = todayClickVal;
		fadeIn(todayMoney);
		fadeIn(todayClicks);
		if (debug)
			log("Today: "+todayEarnings+"\n\n\n");
	} else if (urlObj.location==yesterdayURL) {
		yesterdayClickVal = trim(clicks);
		yesterdayEarnings = trim(earnings);
		yesterdayMoney.data = yesterdayEarnings;
		yesterdayClicks.data = yesterdayClickVal;
		fadeIn(yesterdayMoney);
		fadeIn(yesterdayClicks);
		if (debug)
			log("Yesterday: "+yesterdayEarnings+"\n\n\n");
	} else if (urlObj.location==last7daysURL) {
		last7daysClickVal = trim(clicks);
		last7daysEarnings = trim(earnings);
		sevendaysMoney.data = last7daysEarnings;
		sevendaysClicks.data = last7daysClickVal;
		fadeIn(sevendaysMoney);
		fadeIn(sevendaysClicks);
		if (debug)
			log("7 days: "+last7daysEarnings+"\n\n\n");
	} else if (urlObj.location==thismonthURL) {
		thismonthClickVal = trim(clicks);
		thismonthEarnings = trim(earnings);
		thisMonthMoney.data = thismonthEarnings;
		thisMonthClicks.data = thismonthClickVal;
		fadeIn(thisMonthMoney);
		fadeIn(thisMonthClicks);
		if (debug)
			log("This month: "+thismonthEarnings+"\n\n\n");
	} else if (urlObj.location==lastmonthURL) {
		lastmonthClickVal = trim(clicks);
		lastmonthEarnings = trim(earnings);
		lastMonthMoney.data = lastmonthEarnings;
		lastMonthClicks.data = lastmonthClickVal;
		fadeIn(lastMonthMoney);
		fadeIn(lastMonthClicks);
		if (debug)
			log("Last month: "+lastmonthEarnings+"\n\n\n");
	} else if (urlObj.location==sincelastpaymentURL) {
		sincelastpaymentClickVal = trim(clicks);
		sincelastpaymentEarnings = trim(earnings);
		lastpayMoney.data = sincelastpaymentEarnings;
		lastpayClicks.data = sincelastpaymentClickVal;
		fadeIn(lastpayMoney);
		fadeIn(lastpayClicks);
		if (debug)
			log("Last pay: "+sincelastpaymentEarnings+"\n\n\n");
	} else {
		if (debug)
			log("URL error"+"\n\n\n");
	}

}

/* This function parses the value of total earnings (and clicks)  at the given page.
This function is executed when a page is fetched. */
function parseEarnings(urlObj) {

	if (debug)
		log("Cookies when reading "+urlObj.location+":\n"+cookies+"\n\n\n");

	// Get time when page is read and store it
	var d = new Date(); // Time now
	lastPageReadTime = d.getTime(); // milliseconds since 1970-01-01
		
	// Read page we got
	var contents = urlObj.responseData;
	
	// Search a pattern "Page impressions" which is in the page if there is data available
	var begin = contents.indexOf("Page impressions");
	
	// From this page, search a pattern, starting from "Page impressions", that tries to find clicks and earnings
	if (begin!=-1 && contents.substring(begin,contents.length).search(earningsRegExp)!=-1) {
		
		// Get clicks and earnings from regular expression searched, RegExp.$1 would be pageviews but we don't want that
		var clicks = RegExp.$2;
		var earnings = RegExp.$3;
	
		// Set new values
		setClicksAndEarnings(urlObj, clicks, earnings);
		
	} 
	
	/* If parsing the earnigns has failed, try to search if the data is just not available.
	This happens if the day has changed and no data of earnings is not yet available.
	Also happens, if user has not yet put ads on his site and since no data is available. */
		
	else if (contents.indexOf("No data available")!=-1) {
				
		// No data is available
		var clicks = "-";
		var earnings = "-";
	
		if (debug) {
			//log("Page after no data available:\n"+contents+"\n\n\n");
		}
		
		// Set new values
		setClicksAndEarnings(urlObj, clicks, earnings);
	
	} 
	
	/*The pattern was not found. This means that login has expired and we must login again 
	(or maybe Google has changed web page structure which is BAD BAD) */

	else {
				
		if (debug) {
			log("Error! Cancelling all fetches and logging into AdSense again"+"\n\n\n");
			log("Cookies after parse error:\n"+cookies+"\n\n\n");
			log("Page after parse error:\n"+contents+"\n\n\n");
		}

		// Cancel all fetches because they will likely also fail
		todayURLObj.cancel();
		yesterdayURLObj.cancel();
		last7daysURLObj.cancel();
		thismonthURLObj.cancel();
		lastmonthURLObj.cancel();
		sincelastpaymentURLObj.cancel();	
		
		/* If parse (finding the pattern) is failed two times in a row (so parseFailed was true), 
		show a message that we could not update data. This may be that Google has changed the page structure.
		Otherwise (first parse error) login again.*/
		if (parseFailed) {
			afterError();
			lastUpdated.data = widget.getLocalizedString("parseError");			
			parseFailed = false;
		} else {
			parseFailed = true;
			adsenseLogin();
		}
	}
	
	// Lower the counter
	updateCount--;
	
	// If all earning values have been read, lets update the last updated field
	if (updateCount==0) {
	
		if (debug)
			log("Last page fetched at:\n"+d+"\n\n\n");
		
		parseFailed = false;
		updateLastupdated();
	}
	
}


/* This function fetches the page of the given URL object. */
function readPage(urlObj) {
	
	// Set cookies
	if (cookies!=null)
		urlObj.setRequestHeader("Cookie", cookies);	
	
	// Fetch page, page is parsed when fetch completed
	urlObj.fetchAsync(parseEarnings); 
}  


/* This function is executed after an error has occured (parsing error or network error).
It just basicly fades is the text objects so they are not left transparent.*/
function afterError() {

	// Fade in all thext alements
	fadeIn(todayMoney);
	fadeIn(todayClicks);
	fadeIn(yesterdayMoney);
	fadeIn(yesterdayClicks);
	fadeIn(sevendaysMoney);
	fadeIn(sevendaysClicks);
	fadeIn(thisMonthMoney);
	fadeIn(thisMonthClicks);
	fadeIn(lastMonthMoney);
	fadeIn(lastMonthClicks);
	fadeIn(lastpayMoney);
	fadeIn(lastpayClicks);

}


/* This function stores the cookies to variable. An array of cookie values got from the server must be
passed as parameter. This function reads the cookie name and value and stores them to variable that
is used to pass these cookie values back to server. Cookies are stored in the following format:
CookieName1=Value1; CookieName2=Value2;CookieName3=Value3...*/
function storeCookies(array) {

	// Iterator
	var i;
	
	// Loop through the whole array and store cookies
	for (i=0; i<array.length; i++) {
		
		/* Split the cookie (which is like the following form: CookieName=Value;Domain=xxxx;Path=xxxxx.
		We are only interested in the first cookie name-value pair which we pass back to server when reading pages. */
		var cookieValues = array[i].split(";");
		
		/* Ignore all cookies where reads "EXPIRED". Also ignore cookies "GA3T" and "GoogleAccountsLocale_session"
		if they are already set. */
		if (cookieValues[0].indexOf("EXPIRED")!=-1 ||
			(cookies!=null && 
			((cookies.indexOf("GA3T")!=-1 && cookieValues[0].indexOf("GA3T")!=-1) ||
			(cookies.indexOf("GoogleAccountsLocale_session")!=-1 && 
			cookieValues[0].indexOf("GoogleAccountsLocale_session")!=-1))))
			continue;
		
		// Store cookie name and value
		if (cookies==null)
			cookies = cookieValues[0];
		else
			cookies = cookies+"; "+cookieValues[0];
	}

}


/* This function reads the AdSense login form and gets the GA3T value
which is required to successful login. This function is executed when the login form
page is fetched.*/
function adsenseFormRead(urlObj) {

	// Read cookies and store them
	var cookieArray = urlObj.getResponseHeaders("Set-Cookie");
	storeCookies(cookieArray);
	
	if (debug)
		log("Cookies when read the form:\n"+cookieArray+"\n\n\n");
		
	// Iterator
	var i;
	
	/* Get the GA3T value from cookies by looping through the cookie array and searching a cookie
	with name GA3T. */
	for (i=0; i<cookieArray.length; i++) {
		if (cookieArray[i].indexOf("GA3T")!=-1) {
			/* Split the cookie string just in case. The cookie is usually GA3T=somevalue 
			but it might have some other parameters which we are not interested in.*/
			var cookieSplit = cookieArray[i].split(";");
			GA3Tparsed = cookieSplit[0];
		}
	}
		
	if (debug) {
		log("GA3T value:\n"+GA3Tparsed+"\n\n\n");
	}	

	// We are ready to post the AdSense login form fieds
	sendAdsenseLoginForm();

}

/* This function begins the AdSense login process.*/
function adsenseLogin() {

	// Set cookies to null, otherwise this variable might get too large when we store the cookie values
	cookies = null;
	
	// Read the login form page
	formURLObj = new URL();
	formURLObj.location = formURL;
	formURLObj.timeout = 20; // set timeout to 20 seconds
	formURLObj.fetchAsync(adsenseFormRead); 

}

/* This function finalizes the login process. Used when logging in successfully with Google account and encountered
a redirection page that is done with "meta http-equiv". */
function finalizeLogin(url) {
	
	// Set the redirect URL page
	var redirURLObj = new URL();
	redirURLObj.location = url;
	redirURLObj.timeout = 20; // set timeout to 20 seconds
	
	// Set cookies
	if (cookies!=null)
		redirURLObj.setRequestHeader("Cookie", cookies);

	// Fetch page and finish login. When page is fetched the login is (hopefully) completed and we can read the clicks and earnings
	redirURLObj.fetchAsync(readAllEarnings); 

}


/*
=================================================================================
This function does the actual login process to AdSense. It goes the same way as you do yourself when 
you use your web browser and login to AdSense. This function just automates this login process.

The login process is the following (as far as based on my reengineering):
1. First go to url https://www.google.com/accounts/ServiceLoginBox where the actual login form is.
The actual url used is a bit longer with some extra variables (see variable formURL) so that we are automatically redirected to 
the main page of AdSense instead of the main page of Google accounts. From this page we get the 
auto generated GA3T value from Google which is required to successfull login.
2. Post the login form fields (like username and password and some other hidden fields) to url
https://www.google.com/accounts/ServiceLoginBoxAuth.
3. If the posted fields were correct, Google returns a page where we can find an activation url.
By clicking this url, Google does some kind of authentication process and finally redirects us
automatically to the main page of AdSense. When logging in with Google account (so called unified login), 
the activation url is like https://www.google.com/accounts/CheckCookie?xxxx.
And when logging in with the traditional AdSense account (used only in AdSense, not other Google services),
the activation url is like https://www.google.com/adsense/adsauth?xxxxxxx.
Usually a web browser automatically redirects itself to this activation url with javascript but
we have to manually search the activation url and fetch the page because widgets do not care of redirects made
with javascript (only redirects in HTTP headers). 

So by clicking the activation url, Google redirects us automatically to main page of AdSense
(it uses the HTTP headers and URL object redirects itself automatically to these urls).
The redirect process is the following after clicking the activaton url:
-->/adsense/gaiaauth?auth=xxxxxxxxx
--> /adsense/home?gsessionid=xxxxxxxxx
--> /adsense/report/overview

4. After the login is completed, user can read information of his AdSense account from URLs such as
https://www.google.com/adsense/report/overview?timePeriod=today. User must pass correct cookies when reading these pages.
Passing correct cookies are crucial.

NOTE: At the end of October 2007 Google changed the login process a little bit when using the so
called unified login (logging in with Google account). There was added an extra redirection page that
does the redirection with HTML's "meta http-equiv". Normally the web browser should notice this redirection
but this script doesn't since the redirection isnt' done with HTTP headers, so the script needs to 
"click" the redirect URL to finalize the login process.

=================================================================================
*/
 
function sendAdsenseLoginForm() {

   // Create new HTTP connection
   xmlhttp = new XMLHttpRequest();

   
	// Set cookies
	if (cookies!=null)
		xmlhttp.setRequestHeader('Cookie', cookies);
   
   // Set header field to that is required when posting data
   xmlhttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');

   // Set other header fields
   xmlhttp.setRequestHeader('Referer', formURL);
   xmlhttp.setRequestHeader('User-Agent', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727; .NET CLR 1.1.4322)');   
   
   // Specify function that is executed when the state of the connection is changed
   xmlhttp.onreadystatechange=loginComplete;

  // Open connection (async) and send data to server required to login
  xmlhttp.open("POST", loginURL, true);
  xmlhttp.send("ltmpl=login"+
  "&continue=https%3A%2F%2Fwww.google.com%2Fadsense%2Fgaiaauth"+
  "&followup=https%3A%2F%2Fwww.google.com%2Fadsense%2Fgaiaauth"+
  "&service=adsense"+
  "&nui=15"+
  "&fpui=3"+
  "&ifr=true"+
  "&rm=hide"+
  "&hl=en_US"+
  "&alwf=true"+
  "&"+GA3Tparsed+
  "&Email="+login+
  "&Passwd="+passwd+
  "&null=Sign+in");	
	
	if (debug)
		log("Login sent..."+"\n\n\n");

	// Show user that login is in process
	lastUpdated.data = widget.getLocalizedString("loggingIn");
		
    // This function is called when the state of the connection is changed
  function loginComplete() {
	
		// State 4 represents the state that we have got response from the server. Ignore other states.
		if (xmlhttp.readyState==4) {

			/* 
			 *We have received an aswer from the server. Check that was the loading of the page successfull (200 OK).
			We might also get a response that the page is not found (404 Not Found).
			*/
			if (xmlhttp.status==200) {
			
				// Read the response page and store cookies
				var response=xmlhttp.responseText;				
				storeCookies(xmlhttp.getResponseHeader("Set-Cookie"));

				if (debug) {
					log("Headers after POST:\n"+xmlhttp.getAllResponseHeaders()+"\n\n\n");
					log("Cookies after POST:\n"+cookies+"\n\n\n");
					//log("Body after POST:\n"+response+"\n\n\n");
				}
			
			// Search the activation URL from the page
			if (response.search(redirectUrl)!=-1) {
				
					// Read activation URL and replace HTML ampersand marks
					var redirURL = RegExp.$1;
					redirURL = redirURL.replace(/&amp;/g,"&");
					
					if (debug) {
						log("Redir URL:\n"+redirURL+"\n\n\n");
					}					
					
					// Fetch the redirect URL page
					redirURLObj = new URL();
										
					//redirURLObj.autoRedirect = false;   // just for debug use, URL object uses auto redirect by default
					redirURLObj.location = redirURL;
					redirURLObj.timeout = 20; // set timeout to 20 seconds
					
					// Set cookies
					if (cookies!=null)
						redirURLObj.setRequestHeader("Cookie", cookies);

					// Fetch page and finish login. When page is fetched the login is (hopefully) completed and we can read the clicks and earnings
					redirURLObj.fetchAsync(readAllEarnings); 

			} else {
			
				// If the activation URL is not found, show error. Password may be wrong or Google has changed page structure.
				afterError();
				lastUpdated.data = widget.getLocalizedString("loginError_short");
				alert(widget.getLocalizedString("loginError"));

				if (debug)
					log("Redir URL error"+"\n\n\n");
			}			

			
			} else {
				// If we get something else than 200 OK, the login page is not found (probably network error) 
				loginNotCompleted=true;
				afterError();
				lastUpdated.data = widget.getLocalizedString("network_error");
			}
		} 
	}

      
}


/* This function removes all the leading and trailing whitespaces from given string */
function trim(stringToTrim) {
	return stringToTrim.replace(/^\s+|\s+$/g,"");
}


/* This function executes some commands when widget is closing */
function closeAll() {

	// Cancel all fetches before closing (otherwise an error may occur during closing)
	
	todayURLObj.cancel();
	yesterdayURLObj.cancel();
	last7daysURLObj.cancel();
	thismonthURLObj.cancel();
	lastmonthURLObj.cancel();
	sincelastpaymentURLObj.cancel();	

	formURLObj.cancel();
	redirURLObj.cancel();

	// XMLHTTPRequest uses abort() function instead of cancel().
	xmlhttp.abort();

}